package lifecam

import (
	"errors"
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"path/filepath"
	"sort"
	"strconv"
	"strings"

	"github.com/icza/mjpeg"
)

const (
	fps = 1
)

type Frame struct {
	Session       int
	Frame         int
	Path          string
	Width, Height int
}

func (f *Frame) String() string {
	return fmt.Sprintf("[FRAME] S: %02d F: %04d P: %s",
		f.Session, f.Frame, f.Path)
}

func FrameFromPath(path string) (*Frame, error) {
	names := strings.Split(path, "-")
	if len(names) != 3 {
		return nil, fmt.Errorf("lifecam: %s isn't a lifecam frame", path)
	}

	f := &Frame{
		Path: path,
	}

	var err error
	f.Session, err = strconv.Atoi(names[1])
	if err != nil {
		return nil, err
	}

	names[2] = strings.TrimSuffix(names[2], filepath.Ext(names[2]))
	f.Frame, err = strconv.Atoi(names[2])
	if err != nil {
		return nil, err
	}

	f.Width, f.Height, err = getImageSize(path)
	if err != nil {
		return nil, err
	}

	return f, nil
}

var (
	ErrEmptyFrameList    = errors.New("lifecam: empty framelist")
	ErrFrameListSequence = errors.New("lifecam: framelist out of sequence")
	ErrFrameListSession  = errors.New("lifecam: mismatched sessions in framelist")
	ErrFrameDimensions   = errors.New("lifecam: frames have mismatched dimensions")
)

type FrameList []*Frame

func (fl FrameList) Check() error {
	if len(fl) == 0 {
		return ErrEmptyFrameList
	}

	sort.Sort(fl)

	for i := 1; i < len(fl); i++ {
		if fl[i].Frame <= fl[i-1].Frame {
			return ErrFrameListSequence
		}

		if fl[i].Session != fl[i-1].Session {
			return ErrFrameListSession
		}

		if fl[i].Width != fl[i-1].Width {
			return ErrFrameDimensions
		}

		if fl[i].Height != fl[i-1].Height {
			return ErrFrameDimensions
		}
	}

	return nil
}

func (fl FrameList) String() string {
	if len(fl) == 0 {
		return "empty framelist"
	}

	return fmt.Sprintf("S: %d F: %d", fl[0].Session, len(fl))
}

func (fl FrameList) Len() int {
	return len(fl)
}

func (fl FrameList) Less(i, j int) bool {
	return fl[i].Frame < fl[j].Frame
}

func (fl FrameList) Swap(i, j int) {
	fl[i], fl[j] = fl[j], fl[i]
}

func (fl FrameList) Name() (string, error) {
	fi, err := os.Stat(fl[0].Path)
	if err != nil {
		panic(err.Error())
	}

	ts := fi.ModTime().Format("2006-01-02_1534")
	return fmt.Sprintf("lifecam_%s_%02d_%04d.avi",
		ts, fl[0].Session, len(fl)), nil
}

func LoadDir(imageDir string, frameMap map[int]FrameList) (map[int]FrameList, error) {
	if frameMap == nil {
		frameMap = map[int]FrameList{}
	}

	dir, err := os.ReadDir(imageDir)
	if err != nil {
		return nil, err
	}

	for _, dirent := range dir {
		path := filepath.Join(imageDir, dirent.Name())
		fi, err := os.Stat(path)
		if err != nil {
			return nil, err
		}

		if !fi.Mode().IsRegular() {
			continue
		}

		f, err := FrameFromPath(path)
		if err != nil {
			return nil, err
		}

		frameMap[f.Session] = append(frameMap[f.Session], f)
	}

	return frameMap, nil
}

func (fl FrameList) ToMJPEG(outPath string) error {
	err := fl.Check()
	if err != nil {
		return err
	}

	width := int32(fl[0].Width)
	height := int32(fl[0].Height)
	aviFile, err := mjpeg.New(outPath, width, height, fps)
	if err != nil {
		return err
	}
	defer aviFile.Close()

	for _, frame := range fl {
		img, err := ioutil.ReadFile(frame.Path)
		if err != nil {
			return err
		}

		if err = aviFile.AddFrame(img); err != nil {
			return err
		}
	}

	log.Printf("encoded %s", outPath)
	return aviFile.Close()
}

type Package struct {
	Frames FrameList
	Video  string
}

func (fl FrameList) Package() (*Package, error) {
	path, err := fl.Name()
	if err != nil {
		return nil, err
	}

	err = fl.ToMJPEG(path)
	if err != nil {
		return nil, err
	}

	return &Package{
		Frames: fl,
		Video:  path,
	}, nil
}
