package lifecam

import (
	"archive/zip"
	"io"
	"log"
	"os"
	"path/filepath"
	"strings"
)

func zipName(p string, top bool) string {
	if top {
		return filepath.Base(p)
	}
	return filepath.Join("frames", filepath.Base(p))
}

func pathHeader(p string, top bool) (*zip.FileHeader, error) {

	fi, err := os.Stat(p)
	if err != nil {
		return nil, err
	}

	h, err := zip.FileInfoHeader(fi)
	if err != nil {
		return nil, err
	}

	h.Name = zipName(p, top)
	h.Method = zip.Deflate
	return h, nil
}

type zipFile struct {
	path  string
	file  *os.File
	zw    *zip.Writer
	err   error
	clean bool

	zipped []string
}

func (zf *zipFile) Close() error {
	zf.zw.Close()
	err := zf.file.Close()
	if zf.err != nil {
		os.Remove(zf.path)
	}
	if zf.clean {
		zf.Clean()
	}
	return err
}

func (zf *zipFile) Clean() (err error) {
	targets := zf.zipped[:]
	for i := range targets {
		err = os.Remove(targets[i])
		if err != nil {
			zf.zipped = zf.zipped[i:]
			break
		}
		log.Println("removed", targets[i])
		zf.zipped = zf.zipped[1:]
	}

	if len(zf.zipped) != 0 {
		log.Printf("failed to remove %s",
			strings.Join(zf.zipped, " "))
		return err
	}
	return nil
}

func (zf *zipFile) addPath(path string, top bool) error {
	var (
		f *os.File
		h *zip.FileHeader
		w io.Writer
	)

	h, zf.err = pathHeader(path, top)
	if zf.err != nil {
		return zf.err
	}
	log.Println("zipping", h.Name)

	w, zf.err = zf.zw.CreateHeader(h)
	if zf.err != nil {
		return zf.err
	}

	f, zf.err = os.Open(path)
	if zf.err != nil {
		return zf.err
	}
	defer f.Close()

	_, zf.err = io.Copy(w, f)
	if zf.err != nil {
		return zf.err
	}

	zf.zipped = append(zf.zipped, path)
	return nil
}

func (zf *zipFile) AddVideo(path string) error {
	return zf.addPath(path, true)
}

func (zf *zipFile) AddFrame(path string) error {
	return zf.addPath(path, false)
}

func newZipFile(path string, clean bool) (*zipFile, error) {
	zf := &zipFile{path: path, clean: clean}
	zf.file, zf.err = os.Create(path)
	if zf.err != nil {
		return nil, zf.err
	}

	zf.zw = zip.NewWriter(zf.file)
	return zf, nil
}

func PackageZip(p *Package, clean bool) (err error) {
	path := strings.TrimSuffix(p.Video,
		filepath.Ext(p.Video)) + ".zip"
	zf, err := newZipFile(path, clean)
	if err != nil {
		return err
	}
	defer zf.Close()

	err = zf.AddVideo(p.Video)
	if err != nil {
		return err
	}

	for _, frame := range p.Frames {
		err = zf.AddFrame(frame.Path)
		if err != nil {
			return err
		}
	}

	zf.zw.Close()

	if clean {
		err = zf.Clean()
	}

	return err
}
